/*
 * @author Lenidas de Oliveira Brando
 * 
 * Exemplo didtico para introduo  "programao orientada a objetos" (na linguagem Java)
 * Constri aleatoriamente (random) um vetor e lista na tela
 *
 * Ordena.java -> Desenha.java
 *
 */

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
//import java.io.*; // para o System.in.read()

public class Ordena extends Applet implements ActionListener {

 static final int tam_Vetor     =  20;
 static final double max_double =  20;
 final static long DELAY        =  10;
 final static long steps        =   4;

 // Object -+- Component -+- <Container> -+- Panel - Applet
 //         |             |               |
 //         .             .               +- Window -+- Dialog - FileDialog
 //         .             .                          |
 //         .             .                          +- Frame
 //
 //Frame JanelaPrincipal;
 //Window JanelaPrincipal;
 Container JanelaPrincipal;   

 Desenha desenha;

 int Vet[];

 boolean applet = false; // sinaliza se  ou no "applet" para selecionar construtores corretos
 boolean frame  = false; // sinaliza que no desejamos abrir um Frame a partir de um "applet"
                         // (neste caso abrimos um Panel dentro do "applet")

 // Usado para atrasar a atualizao de cada passo de ordenao (para efeitos visuais)
 private void Retardo(int i, int j) {
     try {
      if (Thread.currentThread().isInterrupted())
         throw new InterruptedException();
      desenha.pos(i,j);    // posiciona uma barra longa
      desenha.repaint(0);
      Thread.sleep(steps * DELAY);
     } catch (InterruptedException ie) { }
 }

 // Ordena o vetor Vet[] usando "Bolha"
 void OrdenaVet () {
     int i,j, i_min, aux, tam = Vet.length;

     for (i=0; i<tam-1; i++) {
	 i_min = i;
	 for (j=i+1; j<tam; j++)
	     if (Vet[j] < Vet[i_min]) i_min = j;
	 aux = Vet[i];
	 Vet[i] = Vet[i_min];
	 Vet[i_min] = aux;
         Retardo(i,i_min);
	 desenha.pinta(); // desenha cada alterao no "offscreen" e depois move tudo para a tela
         }
     //Vet = V;
 }

 void inicia_Vetor () {
     int i;
     //int[] values = ArrayUtil.randomIntArray(30, 300);
     Vet = new int [tam_Vetor]; 
     for (i=0; i<tam_Vetor; i++) {
	 Vet[i] = (int)(max_double*Math.random())%tam_Vetor;
     }
 }

 //  chamado aps o "Applet" ser instanciado
 public void init (){ // sempre que for carregado por applet, entra aqui
     applet = true;
     this.montaJanela(); //  possvel fazer isso em funo do "init()" 
                         // s ser chamado aps criar a instncia da classe principal
 }

 //
 // Precisa do construtor: 
 //  caso contrrio o "main" teria as chamadas "inicia_Vetor();" e "new Desenha( Vet );"
 //  ambas resultando em "static reference to X in class Ordena", sendo X respectivamente,
 // "method void inicia_Vetor()" e "nonstatic variable Vet"
 //
 public Ordena () {
     }
   
 private void montaJanela () {	
     Label L = new Label("Desenha lista de valores");
     Color cor_Fundo = new Color(30, 100, 180);
     Panel p = new Panel();
     Button b_ord = new Button("Ord");
     Button b_nov = new Button("Novo");
     Button b_fim = new Button("Fim");
    
     if (applet && !frame) {
        JanelaPrincipal = this; //
        }    
     else
        JanelaPrincipal = new Frame("Desenha lista de valores");

     JanelaPrincipal.setBackground( cor_Fundo );
     JanelaPrincipal.setLayout(new BorderLayout());

     b_ord.addActionListener(this);
     b_nov.addActionListener(this);
     b_fim.addActionListener(this);
     p.add(b_ord);
     p.add(b_nov);
     if (!applet) p.add(b_fim);
     inicia_Vetor();
     desenha = new Desenha( this, Vet );
     desenha.vetor( Vet );

     JanelaPrincipal.add(desenha,"Center");
     JanelaPrincipal.add(p,"South");
     JanelaPrincipal.add(L,"North");
     JanelaPrincipal.setSize(400,200);
     JanelaPrincipal.setVisible(true); //show();
 }

 public void actionPerformed(ActionEvent e) {
        String s = e.getActionCommand();
        if ( s.equals ("Fim") ) {
	   if (! applet) System.exit(0);
	   //else ((Applet)JanelaPrincipal).destroy(); // no existe mtodo "dispose" na classe abstrata Container
	   else this.destroy(); // no existe mtodo "dispose" na classe abstrata Container
	}
        else
        if ( s.equals ("Ord") ) {
            OrdenaVet( );
	}
        else
        if ( s.equals ("Novo") ) {
           inicia_Vetor();
           desenha.vetor( Vet );
           desenha.repaint();
	}
 }

 public static void main(String[] args) {
     Ordena janela = new Ordena(); //  necessrio instanciar "Ordena", pois "main"  esttico
     janela.montaJanela();         // e por isso fazer "this.montaJanela()" d erro
 }

}
